package com.dragon.tj.portal.auth.config;

import com.dragon.tj.portal.auth.common.constants.AuthType;
import com.dragon.tj.portal.auth.module.cas.MyCasAuthenticationEntryPoint;
import com.dragon.tj.portal.auth.module.cas.MySimpleUrlAuthenticationSuccessHandler;
import com.dragon.tj.portal.auth.properties.AppProperties;
import com.dragon.tj.portal.auth.properties.CasProperties;
import com.dragon.tj.portal.auth.service.JwtTokenAuthenticationFilter;
import com.dragon.tj.portal.auth.service.JwtTokenLogoutSuccessHandler;
import com.dragon.tj.portal.auth.service.MyUserDetailsByNameServiceWrapper;
import com.dragon.tj.portal.auth.service.MyUserDetailsService;
import com.dragon.tj.portal.auth.service.TokenService;
import org.jasig.cas.client.validation.Cas20ServiceTicketValidator;
import org.jasig.cas.client.validation.TicketValidator;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.ProviderManager;
import org.springframework.security.cas.ServiceProperties;
import org.springframework.security.cas.authentication.CasAuthenticationProvider;
import org.springframework.security.cas.web.CasAuthenticationFilter;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationFailureHandler;

@Configuration
@ConditionalOnProperty(name = "app.auth-type", havingValue = AuthType.CAS, matchIfMissing = true)
@EnableConfigurationProperties(CasProperties.class)
public class WebSecurityCasConfig {

    private final AppProperties appProperties;
    private final CasProperties casProperties;
    private final JwtTokenAuthenticationFilter jwtTokenAuthenticationFilter;
    private final MyUserDetailsService userDetailsService;
    private final MySimpleUrlAuthenticationSuccessHandler mySimpleUrlAuthenticationSuccessHandler;
    private final TokenService tokenService;

    public WebSecurityCasConfig(
            AppProperties appProperties, CasProperties casProperties,
            JwtTokenAuthenticationFilter jwtTokenAuthenticationFilter,
            MyUserDetailsService userDetailsService,
            MySimpleUrlAuthenticationSuccessHandler mySimpleUrlAuthenticationSuccessHandler,
            TokenService tokenService) {
        this.appProperties = appProperties;
        this.casProperties = casProperties;
        this.jwtTokenAuthenticationFilter = jwtTokenAuthenticationFilter;
        this.userDetailsService = userDetailsService;
        this.mySimpleUrlAuthenticationSuccessHandler = mySimpleUrlAuthenticationSuccessHandler;
        this.tokenService = tokenService;
    }

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http
                // CSRF禁用，因为不使用session
                .csrf().disable()
                // Enable CORS
                .cors()
                .and()
                .authorizeRequests()
                .antMatchers(WebSecurityConfig.WHITE_LIST.toArray(new String[0])).permitAll()
                .anyRequest().authenticated()
                .and()
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                // 因为CasAuthenticationFilter仅拦截/sso/login，所以未认证前访问其他url失败时都走到这个兜底的exception处理
                .exceptionHandling(exceptions -> exceptions.authenticationEntryPoint(authenticationEntryPoint()))
                // CAS SSO
                .addFilter(casAuthenticationFilter())
                .addFilterBefore(jwtTokenAuthenticationFilter, CasAuthenticationFilter.class)
                .logout()
                .logoutUrl(appProperties.getLogoutUrl())
                .logoutSuccessHandler(jwtTokenLogoutSuccessHandler());
        return http.build();
    }

    public AuthenticationEntryPoint authenticationEntryPoint() {
        MyCasAuthenticationEntryPoint casAuthenticationEntryPoint = new MyCasAuthenticationEntryPoint();
        casAuthenticationEntryPoint.setLoginUrl(casProperties.getLoginUrl());
        casAuthenticationEntryPoint.setServiceProperties(serviceProperties());
        return casAuthenticationEntryPoint;
    }

    public CasAuthenticationFilter casAuthenticationFilter() {
        CasAuthenticationFilter filter = new CasAuthenticationFilter();
        filter.setFilterProcessesUrl(appProperties.getFilterUrl());

        CasAuthenticationProvider casAuthenticationProvider = casAuthenticationProvider(userDetailsService);
        filter.setAuthenticationManager(new ProviderManager(casAuthenticationProvider));

        mySimpleUrlAuthenticationSuccessHandler.setDefaultTargetUrl(appProperties.getTargetUrl());
        filter.setAuthenticationSuccessHandler(mySimpleUrlAuthenticationSuccessHandler);
        filter.setAuthenticationFailureHandler(new SimpleUrlAuthenticationFailureHandler(appProperties.getFailureUrl()));

        return filter;
    }

    public CasAuthenticationProvider casAuthenticationProvider(UserDetailsService userDetailsService) {
        CasAuthenticationProvider provider = new CasAuthenticationProvider();
        provider.setAuthenticationUserDetailsService(new MyUserDetailsByNameServiceWrapper<>(userDetailsService));
        provider.setServiceProperties(serviceProperties());
        provider.setTicketValidator(ticketValidator());
        provider.setKey("key");
        return provider;
    }

    public ServiceProperties serviceProperties() {
        ServiceProperties serviceProperties = new ServiceProperties();
        serviceProperties.setService(appProperties.getServiceUrl());
        return serviceProperties;
    }

    private TicketValidator ticketValidator() {
        return new Cas20ServiceTicketValidator(casProperties.getBaseUrl());
    }

    public JwtTokenLogoutSuccessHandler jwtTokenLogoutSuccessHandler() {
        return new JwtTokenLogoutSuccessHandler(appProperties.getHomeUrl(), casProperties.getLogoutUrl(), tokenService);
    }

}
