package com.dragon.tj.portal.auth.config;

import com.dragon.tj.portal.auth.module.hmac.HmacAuthenticationFilter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityCustomizer;
import org.springframework.security.config.annotation.web.configurers.CsrfConfigurer;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationEntryPointFailureHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

@Configuration
@EnableWebSecurity
public class WebSecurityConfig {

    public static final List<String> WHITE_LIST;

    static {
        // 白名单
        WHITE_LIST = new ArrayList<>();
        WHITE_LIST.add("/test/login");
        WHITE_LIST.add("/file/**");
    }

    private final JdbcTemplate jdbcTemplate;

    public WebSecurityConfig(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    @Bean
    public WebSecurityCustomizer webSecurityCustomizer() {
        return (web) -> web.ignoring()
                // 认证成功后才会忽略
                .antMatchers("/resources/**");
    }

    @Bean
    public SecurityFilterChain apiSecurityFilterChain(HttpSecurity http) throws Exception {
        http
                .antMatcher("/api/**")
                // CSRF禁用，因为不使用session
                .csrf(CsrfConfigurer::disable)
                .authorizeRequests(authorizeRequests -> authorizeRequests.anyRequest().authenticated())
                .sessionManagement(sessionManagement -> sessionManagement.sessionCreationPolicy(SessionCreationPolicy.STATELESS))
                // hmacFilter情况下大概率用不到这个exceptionHandling
                .exceptionHandling(exceptions -> exceptions.authenticationEntryPoint(apiAuthenticationEntryPoint()))
                // 内置filters已经排序 FilterOrderRegistration.FilterOrderRegistration()
                .addFilterBefore(hmacAuthenticationFilter(), UsernamePasswordAuthenticationFilter.class);
        return http.build();
    }

    public AuthenticationEntryPoint apiAuthenticationEntryPoint() {
        return (request, response, authException) -> {
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            response.setCharacterEncoding(StandardCharsets.UTF_8.name());
            response.setContentType(MediaType.APPLICATION_JSON.toString());
            response.getWriter().write("{\"msg:\":\"认证失败：" + authException.getMessage() + "\"}");
        };
    }

    public HmacAuthenticationFilter hmacAuthenticationFilter() {
        // HmacAuthenticationFilter filter = new HmacAuthenticationFilter("/api/token");
        HmacAuthenticationFilter filter = new HmacAuthenticationFilter("/api/**");
        filter.setJdbcTemplate(jdbcTemplate);
        filter.setAuthenticationFailureHandler(new AuthenticationEntryPointFailureHandler(apiAuthenticationEntryPoint()));
        filter.setAuthenticationSuccessHandler((request, response, authentication) ->
                SecurityContextHolder.getContext().setAuthentication(authentication)
        );
        return filter;
    }

}
