package com.dragon.tj.portal.auth.config;

import com.dragon.tj.portal.auth.common.constants.AuthType;
import com.dragon.tj.portal.auth.module.cas.MyCasAuthenticationEntryPoint;
import com.dragon.tj.portal.auth.module.cas.MySimpleUrlAuthenticationSuccessHandler;
import com.dragon.tj.portal.auth.module.zerotrust.ZeroTrustAuthenticationFilter;
import com.dragon.tj.portal.auth.module.zerotrust.ZeroTrustAuthenticationProvider;
import com.dragon.tj.portal.auth.properties.AppProperties;
import com.dragon.tj.portal.auth.properties.ZeroTrustProperties;
import com.dragon.tj.portal.auth.service.JwtTokenAuthenticationFilter;
import com.dragon.tj.portal.auth.service.JwtTokenLogoutSuccessHandler;
import com.dragon.tj.portal.auth.service.MyUserDetailsService;
import com.dragon.tj.portal.auth.service.TokenService;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.ProviderManager;
import org.springframework.security.cas.ServiceProperties;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationFailureHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

@Configuration
@ConditionalOnProperty(name = "app.auth-type", havingValue = AuthType.ZERO_TRUST)
@EnableConfigurationProperties(ZeroTrustProperties.class)
public class WebSecurityZeroTrustConfig {

    private final AppProperties appProperties;
    private final ZeroTrustProperties zeroTrustProperties;
    private final JwtTokenAuthenticationFilter jwtTokenAuthenticationFilter;
    private final MyUserDetailsService userDetailsService;
    private final MySimpleUrlAuthenticationSuccessHandler mySimpleUrlAuthenticationSuccessHandler;
    private final TokenService tokenService;

    public WebSecurityZeroTrustConfig(
            AppProperties appProperties,
            ZeroTrustProperties zeroTrustProperties,
            JwtTokenAuthenticationFilter jwtTokenAuthenticationFilter,
            MyUserDetailsService userDetailsService,
            MySimpleUrlAuthenticationSuccessHandler mySimpleUrlAuthenticationSuccessHandler,
            TokenService tokenService) {
        this.appProperties = appProperties;
        this.zeroTrustProperties = zeroTrustProperties;
        this.jwtTokenAuthenticationFilter = jwtTokenAuthenticationFilter;
        this.userDetailsService = userDetailsService;
        this.mySimpleUrlAuthenticationSuccessHandler = mySimpleUrlAuthenticationSuccessHandler;
        this.tokenService = tokenService;
    }

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http
                // CSRF禁用，因为不使用session
                .csrf().disable()
                // Enable CORS
                .cors()
                .and()
                .authorizeRequests()
                .antMatchers(WebSecurityConfig.WHITE_LIST.toArray(new String[0])).permitAll()
                .anyRequest().authenticated()
                .and()
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                // 因为CasAuthenticationFilter仅拦截/sso/login，所以未认证前访问其他url失败时都走到这个兜底的exception处理
                .exceptionHandling(exceptions -> exceptions.authenticationEntryPoint(authenticationEntryPoint()))
                // HuaWei Zero Trust
                .addFilterBefore(jwtTokenAuthenticationFilter, UsernamePasswordAuthenticationFilter.class)
                .addFilterBefore(zeroTrustAuthenticationFilter(), UsernamePasswordAuthenticationFilter.class)
                .logout()
                .logoutUrl(appProperties.getLogoutUrl())
                .logoutSuccessHandler(jwtTokenLogoutSuccessHandler());
        return http.build();
    }

    public AuthenticationEntryPoint authenticationEntryPoint() {
        MyCasAuthenticationEntryPoint casAuthenticationEntryPoint = new MyCasAuthenticationEntryPoint();
        casAuthenticationEntryPoint.setLoginUrl(zeroTrustProperties.getLoginUrl());
        casAuthenticationEntryPoint.setServiceProperties(serviceProperties());
        return casAuthenticationEntryPoint;
    }

    public ServiceProperties serviceProperties() {
        ServiceProperties serviceProperties = new ServiceProperties();
        serviceProperties.setService(appProperties.getServiceUrl());
        return serviceProperties;
    }

    public ZeroTrustAuthenticationFilter zeroTrustAuthenticationFilter() {
        ZeroTrustAuthenticationFilter filter = new ZeroTrustAuthenticationFilter(appProperties.getFilterUrl());
        filter.setAuthenticationManager(new ProviderManager(new ZeroTrustAuthenticationProvider(userDetailsService)));
        mySimpleUrlAuthenticationSuccessHandler.setDefaultTargetUrl(appProperties.getTargetUrl());
        filter.setAuthenticationSuccessHandler(mySimpleUrlAuthenticationSuccessHandler);
        filter.setAuthenticationFailureHandler(new SimpleUrlAuthenticationFailureHandler(appProperties.getFailureUrl()));
        return filter;
    }

    public JwtTokenLogoutSuccessHandler jwtTokenLogoutSuccessHandler() {
        return new JwtTokenLogoutSuccessHandler(appProperties.getHomeUrl(), zeroTrustProperties.getLogoutUrl(), tokenService);
    }

}
