package com.dragon.tj.portal.auth.module.hmac;

import org.apache.commons.codec.digest.HmacAlgorithms;
import org.apache.commons.codec.digest.HmacUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.UUID;

public class HmacAuthenticationFilter extends AbstractAuthenticationProcessingFilter {

    private JdbcTemplate jdbcTemplate;

    public HmacAuthenticationFilter(String defaultFilterProcessesUrl) {
        super(defaultFilterProcessesUrl);
    }

    @Override
    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) throws AuthenticationException {
        String appKey = request.getHeader("appKey");
        if (StringUtils.isBlank(appKey)) {
            throw new BadCredentialsException("未设置appKey");
        }
        // 时间戳验证
        String timestamp = request.getHeader("timestamp");
        if (StringUtils.isBlank(timestamp)) {
            throw new BadCredentialsException("未设置timestamp");
        }
        // 大于5分钟，非法请求
        long diff = System.currentTimeMillis() - Long.parseLong(timestamp);
        if (Math.abs(diff) > 1000 * 60 * 5) {
            throw new BadCredentialsException("请求超时");
        }
        // 随机字符串，防止重复提交
        String nonce = request.getHeader("nonce");
        if (StringUtils.isEmpty(nonce)) {
            throw new BadCredentialsException("未设置nonce");
        }
        // 验证签名
        String signature = request.getHeader("signature");
        if (StringUtils.isEmpty(nonce)) {
            throw new BadCredentialsException("未设置signature");
        }

        String appSecret;
        try {
            appSecret = jdbcTemplate.queryForObject("select app_secret from app_info where system_number = ?",
                    String.class, appKey);
        } catch (Exception e) {
            throw new BadCredentialsException("查询应用出错，appKey: " + appKey, e);
        }
        if (StringUtils.isEmpty(appSecret)) {
            throw new BadCredentialsException("未查询到该应用，appKey: " + appKey);
        }

        String signPayload = String.format("%s:%s:%s", appKey, timestamp, nonce);
        String signResult = new HmacUtils(HmacAlgorithms.HMAC_SHA_256, appSecret).hmacHex(signPayload);
        if (!signature.equals(signResult)) {
            throw new BadCredentialsException("Signature验证错误");
        }
        // TODO: 2024/7/17 sai nonce重复请求校验

        // authorities参数必填使authenticated为true
        return new UsernamePasswordAuthenticationToken(appKey, signature, null);
    }

    @Override
    protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain, Authentication authResult) throws IOException, ServletException {
        super.successfulAuthentication(request, response, chain, authResult);
        // continue next filter
        chain.doFilter(request, response);
    }

    public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    public static void main(String[] args) {
        System.out.println(UUID.randomUUID().toString().replace("-", ""));
        long l = System.currentTimeMillis();
        System.out.println("timestamp: " + l);
        String appSecret = "ae14ac8a78104a4392920636c40335ca";
        String signPayload = String.format("%s:%s:%s", "xxdj05", l, "412710e34fee4c8ba29a210462be8e6b");
        System.out.println(new HmacUtils(HmacAlgorithms.HMAC_SHA_256, appSecret).hmacHex(signPayload));
    }
}
