package com.dragon.tj.portal.auth.service;

import com.dragon.tj.portal.auth.model.AuthReq;
import com.dragon.tj.portal.auth.model.AuthResult;
import com.dragon.tj.portal.auth.model.RR;
import com.dragon.tj.portal.auth.model.zeroTrust.AuthorizationRequest;
import com.dragon.tj.portal.auth.model.zeroTrust.AuthorizationResponse;
import com.dragon.tj.portal.auth.module.cas.CasAuthorizationClient;
import com.dragon.tj.portal.auth.module.zerotrust.ZeroTrustAuthorizeClient;
import com.dragon.tj.portal.auth.properties.CasProperties;
import com.dragon.tj.portal.auth.properties.ZeroTrustProperties;
import com.dragon.tj.portal.auth.web.entity.SysUser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import retrofit2.Call;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;

/**
 * 用户权限处理
 *
 * @author tienchin
 */
@Component
public class SysPermissionService {
    private static final Logger LOGGER = LoggerFactory.getLogger(SysPermissionService.class);

    private final CasProperties casProperties;
    private final ZeroTrustProperties zeroTrustProperties;
    private final CasAuthorizationClient casAuthorizationClient;
    private final ZeroTrustAuthorizeClient zeroTrustAuthorizeClient;

    public SysPermissionService(
            CasProperties casProperties,
            ZeroTrustProperties zeroTrustProperties,
            CasAuthorizationClient casAuthorizationClient,
            ZeroTrustAuthorizeClient zeroTrustAuthorizeClient) {
        this.casProperties = casProperties;
        this.zeroTrustProperties = zeroTrustProperties;
        this.casAuthorizationClient = casAuthorizationClient;
        this.zeroTrustAuthorizeClient = zeroTrustAuthorizeClient;
    }

    /**
     * 获取数据权限
     *
     * @param user 用户信息
     * @return 数据权限信息
     */
    public Set<String> getDataPermission(SysUser user) {
        return new HashSet<>();
    }

    /**
     * 获取菜单权限
     *
     * @param user 用户信息
     * @return 菜单权限信息
     */
    public List<String> getMenuPermission(SysUser user) {
        // Default Auth: DCUC
        return getMenuPermissionFromDcuc(user.getIdcard());
    }

    private List<String> getMenuPermissionFromDcuc(String idCard) {
        try {
            LOGGER.info("登录用户[{}]调用权限管理服务-功能级鉴权接口", idCard);
            Call<RR<List<AuthResult>>> call = casAuthorizationClient.getAuthFunctions(
                    casProperties.getAppCode(), idCard, new AuthReq(casProperties.getAppCode(), idCard));
            RR<List<AuthResult>> body = call.execute().body();
            if (Objects.nonNull(body)) {
                if ("200".equals(body.getStatusCode())) {
                    List<AuthResult> results = body.getResult();
                    List<String> permissions = extractPermissions(results);
                    LOGGER.info("登录用户[{}]调用权限管理服务-功能级鉴权接口 成功: {}", idCard, permissions);
                    return permissions;
                } else {
                    LOGGER.error("登录用户[{}]调用权限管理服务-功能级鉴权接口 出错: {}", idCard, body.getMessage());
                }
            } else {
                LOGGER.error("登录用户[{}]调用权限管理服务-功能级鉴权接口 出错: 返回为空", idCard);
            }
        } catch (Exception e) {
            LOGGER.error("登录用户[{}]调用权限管理服务-功能级鉴权接口 出错", idCard, e);
        }
        return new ArrayList<>();
    }

    public List<String> getMenuPermissionFromZeroTrust(String userToken, String appToken) {
        try {
            LOGGER.info("登录用户[{}]调用零信任-功能级鉴权接口", userToken);
            Call<AuthorizationResponse> call = zeroTrustAuthorizeClient.getAuthFunctions(
                    new AuthorizationRequest(appToken, zeroTrustProperties.getTaskId()));
            AuthorizationResponse body = call.execute().body();
            if (Objects.nonNull(body)) {
                if (AuthorizationResponse.SUCCESS.equals(body.getStatusCode())) {
                    String results = body.getData().getResult();
                    List<String> permissions = Arrays.asList(results.split(","));
                    LOGGER.info("登录用户[{}]调用零信任-功能级鉴权接口 成功: {}", userToken, permissions);
                    return permissions;
                } else {
                    LOGGER.error("登录用户[{}]调用零信任-功能级鉴权接口 出错: {}", userToken, body.getMessage());
                }
            } else {
                LOGGER.error("登录用户[{}]调用零信任-功能级鉴权接口 出错: 返回为空", userToken);
            }
        } catch (Exception e) {
            LOGGER.error("登录用户[{}]调用零信任-功能级鉴权接口 出错", userToken, e);
        }
        return new ArrayList<>();
    }

    private List<String> extractPermissions(List<AuthResult> results) {
        List<String> permissions = new ArrayList<>();
        if (!CollectionUtils.isEmpty(results)) {
            for (AuthResult result : results) {
                permissions.add(result.getCode());
                List<AuthResult> nextCode = result.getNextCode();
                if (!CollectionUtils.isEmpty(nextCode)) {
                    permissions.addAll(extractPermissions(nextCode));
                }
            }
        }
        return permissions;
    }
}
