package com.dragon.tj.portal.component.redis;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.StringRedisTemplate;

import java.time.Duration;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author huey China.
 * @Description : redis str 工具
 * @Date Created in 2023/6/15 16:41
 */
public class StringCacheUtil {

    StringRedisTemplate redisTemplate;
    HashOperations<String, String, String> hashOperations;

    public StringCacheUtil(StringRedisTemplate strRedisTemplateCommon) {
        this.redisTemplate = strRedisTemplateCommon;
        hashOperations = strRedisTemplateCommon.opsForHash();
    }

    public boolean expire(String key, long secs) {
        {
            if (secs > 0L) {
                this.redisTemplate.expire(key, secs, TimeUnit.SECONDS);
            }

            return true;
        }
    }

    public long expire(String key) {
        return this.redisTemplate.getExpire(key, TimeUnit.SECONDS);
    }

    public boolean hasKey(String key) {
        {
            return this.redisTemplate.hasKey(key);
        }
    }

    public void del(Collection<String> keys) {
        if (CollectionUtils.isNotEmpty(keys)) {

            this.redisTemplate.delete(keys);

        }

    }

    public void del(String... key) {
        if (key != null && key.length > 0) {
            if (key.length == 1) {
                this.redisTemplate.delete(key[0]);
            } else {
                this.redisTemplate.delete(Arrays.asList(key));
            }
        }

    }

    public String get(String key) {
        return key == null ? null : this.redisTemplate.opsForValue().get(key);
    }

    public boolean mset(Map<String, String> map) {
        {
            this.redisTemplate.opsForValue().multiSet(map);
            return true;
        }
    }

    public boolean set(String key, String value) {
        {
            this.redisTemplate.opsForValue().set(key, value);
            return true;
        }
    }

    public boolean set(String key, String value, long secs) {
        {
            if (secs > 0L) {
                this.redisTemplate.opsForValue().set(key, value, secs, TimeUnit.SECONDS);
            } else {
                this.set(key, value);
            }

            return true;
        }
    }

    public boolean set(String key, String value, Duration duration) {
        this.redisTemplate.opsForValue().set(key, value, duration);
        return true;
    }

    public boolean set(String key, String value, long tiemout, TimeUnit timeUnit) {
        this.redisTemplate.opsForValue().set(key, value, tiemout, timeUnit);
        return true;
    }

    public long incr(String key, long delta) {
//        if (delta < 0L) {
//            throw new RuntimeException("递增因子必须大于0");
//        } else {
        return this.redisTemplate.opsForValue().increment(key, delta);
//        }
    }

    public long decr(String key, long delta) {
//        if (delta < 0L) {
//            throw new RuntimeException("递减因子必须大于0");
//        } else {
        return this.redisTemplate.opsForValue().decrement(key, delta);
//        }
    }

    public String hGet(String key, String item) {
        return hashOperations.get(key, item);
    }

    public Map<String, String> hGetAll(String key) {
        return hashOperations.entries(key);
    }

    public Map<String, String> hMGet(String key, Collection<String> items) {
        return hMGet(key, items.stream().collect(Collectors.toList()));
    }

    public Map<String, String> hMGet(String key, List<String> items) {
        if (CollectionUtils.isEmpty(items)) {
            return MapUtils.EMPTY_SORTED_MAP;
        }

        List<String> strings = hashOperations.multiGet(key, items);

        Map<String, String> map = new HashMap<>();

        for (int i = 0; i < items.size(); i++) {
            map.put(items.get(i), strings.get(i));
        }

        return map;
    }

    public boolean hMSet(String key, Map<String, String> map) {
        {
            hashOperations.putAll(key, map);
            return true;
        }
    }

    public boolean hMSet(String key, Map<String, String> map, long secs) {
        {
            hashOperations.putAll(key, map);
            if (secs > 0L) {
                this.expire(key, secs);
            }

            return true;
        }
    }

    public boolean hSet(String key, String item, String value) {
        {
            hashOperations.put(key, item, value);
            return true;
        }
    }

    public void hDel(String key, String... item) {
        hashOperations.delete(key, item);
    }

    public boolean hExists(String key, String item) {
        return hashOperations.hasKey(key, item);
    }

    public double hIncr(String key, String item, double by) {
        return hashOperations.increment(key, item, by);
    }

    public double hDecr(String key, String item, double by) {
        return hashOperations.increment(key, item, -by);
    }

    public long hIncr(String key, String item, long by) {
        return hashOperations.increment(key, item, by);
    }

    public long hDecr(String key, String item, long by) {
        return hashOperations.increment(key, item, -by);
    }

    public Set<String> sMembers(String key) {
        {
            return this.redisTemplate.opsForSet().members(key);
        }
    }

    public boolean sIsMember(String key, String value) {
        {
            return this.redisTemplate.opsForSet().isMember(key, value);
        }
    }

    public long sAdd(String key, String... values) {
        {
            return this.redisTemplate.opsForSet().add(key, values);
        }
    }

    public long sAdd(String key, long secs, String... values) {
        {
            Long count = this.redisTemplate.opsForSet().add(key, values);
            if (secs > 0L) {
                this.expire(key, secs);
            }

            return count;
        }
    }

    /**
     * number of elements in a set
     *
     * @param key
     * @return
     */
    public long sCard(String key) {
        {
            return this.redisTemplate.opsForSet().size(key);
        }
    }

    /**
     * Remove the specified members from the set
     *
     * @param key
     * @param values
     * @return
     */
    public long sRem(String key, String... values) {
        {
            Long count = this.redisTemplate.opsForSet().remove(key, values);
            return count == null ? 0L : count;
        }
    }

    public List<String> lRange(String key, long start, long end) {
        {
            return this.redisTemplate.opsForList().range(key, start, end);
        }
    }

    public long lLen(String key) {
        {
            return this.redisTemplate.opsForList().size(key);
        }
    }

    public String lIndex(String key, long index) {
        {
            return this.redisTemplate.opsForList().index(key, index);
        }
    }

    public String lPop(String key) {
        {
            return this.redisTemplate.opsForList().leftPop(key);
        }
    }

    /**
     * 加到队尾
     *
     * @param key
     * @param value
     * @return
     */
    public boolean lPush(String key, String value) {
        {
            this.redisTemplate.opsForList().leftPush(key, value);
            return true;
        }
    }

    /**
     * 加到队尾
     *
     * @param key
     * @param value
     * @param secs
     * @return
     */
    public boolean lPush(String key, String value, long secs) {
        {
            this.redisTemplate.opsForList().leftPush(key, value);
            if (secs > 0L) {
                this.expire(key, secs);
            }

            return true;
        }
    }

    /**
     * 加到队尾
     *
     * @param key
     * @param value
     * @return
     */
    public boolean lPush(String key, Collection<String> value) {
        {
            this.redisTemplate.opsForList().leftPushAll(key, value);
            return true;
        }
    }

    /**
     * 加到队尾
     *
     * @param key
     * @param value
     * @param secs
     * @return
     */
    public boolean lPush(String key, Collection<String> value, long secs) {
        {
            this.redisTemplate.opsForList().leftPushAll(key, value);
            if (secs > 0L) {
                this.expire(key, secs);
            }

            return true;
        }
    }

    public String rPop(String key) {
        {
            return this.redisTemplate.opsForList().rightPop(key);
        }
    }

    /**
     * 插到队头
     *
     * @param key
     * @param value
     * @return
     */
    public boolean rPush(String key, String value) {
        {
            this.redisTemplate.opsForList().rightPush(key, value);
            return true;
        }
    }

    /**
     * 插到队头
     *
     * @param key
     * @param value
     * @param secs
     * @return
     */
    public boolean rPush(String key, String value, long secs) {
        {
            this.redisTemplate.opsForList().rightPush(key, value);
            if (secs > 0L) {
                this.expire(key, secs);
            }

            return true;
        }
    }

    /**
     * 插到队头
     *
     * @param key
     * @param value
     * @return
     */
    public boolean rPush(String key, Collection<String> value) {
        {
            this.redisTemplate.opsForList().rightPushAll(key, value);
            return true;
        }
    }

    /**
     * 插到队头
     *
     * @param key
     * @param value
     * @param secs
     * @return
     */
    public boolean rPush(String key, Collection<String> value, long secs) {
        {
            this.redisTemplate.opsForList().rightPushAll(key, value);
            if (secs > 0L) {
                this.expire(key, secs);
            }

            return true;
        }
    }

    public boolean lSet(String key, long index, String value) {
        {
            this.redisTemplate.opsForList().set(key, index, value);
            return true;
        }
    }

    /**
     * @param key
     * @param count count > 0: Remove elements equal to element moving from head to tail.
     *              count < 0: Remove elements equal to element moving from tail to head.
     *              count = 0: Remove all elements equal to element.
     * @param value
     * @return
     */
    public long lRem(String key, long count, String value) {
        {
            Long remove = this.redisTemplate.opsForList().remove(key, count, value);
            return remove;
        }
    }

    public Set<String> keys(String keyPattern) {
        {
            Set<String> keysSet = this.redisTemplate.keys(keyPattern);
            return keysSet;
        }
    }

    public List<String> mGet(Set<String> keySet) {
        {
            List<String> objectList = this.redisTemplate.opsForValue().multiGet(keySet);
            return objectList;
        }
    }

    public void convertAndSend(String channel, String message) {
        this.redisTemplate.convertAndSend(channel, message);
    }

    /**
     * @author huey China.
     * @Description : 存在
     * @Date Created in 2023/5/23 16:44
     */
    public boolean exist(String key, String value, Duration duration) {
        return this.redisTemplate.opsForValue().setIfAbsent(key, value, duration);
    }

    /**
     * @author huey China.
     * @Description : 自增
     * @Date Created in 2023/5/23 16:44
     */
    public Long increment(String key) {
        return this.redisTemplate.opsForValue().increment(key);
    }
}
