package com.dragon.tj.portal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.util.SecurityUtils;
import com.dragon.tj.portal.entity.FileInfo;
import com.dragon.tj.portal.entity.PageParam;
import com.dragon.tj.portal.entity.SysLog;
import com.dragon.tj.portal.mapper.FileManageMapper;
import com.dragon.tj.portal.service.FileManageService;
import com.dragon.tj.portal.service.SysLogService;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class FileManageServiceImpl implements FileManageService {

    @Value("${dragon.file.path}")
    private String localPath;

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private FileManageMapper fileManageMapper;

    @Autowired
    private SysLogService sysLogServices;

    @Override
    public FileInfo upload(MultipartFile file, Integer type) {
        String originalFilename = file.getOriginalFilename();

        File dateDir = new File(localPath + type);
        if (!dateDir.exists()) {
            dateDir.mkdirs();
        }

        File newFile = new File(dateDir.getPath(), originalFilename);

        try {
            FileUtils.copyInputStreamToFile(file.getInputStream(), newFile);
            // file.transferTo(newFile); // 相对路径会出错
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        Map<String, String> res = new HashMap<>();
        FileInfo fileInfo = new FileInfo();
        String[] split = originalFilename.split("\\.");
        fileInfo.setFileName(split[0]);
        fileInfo.setExtension(split[1]);
        fileInfo.setSize(file.getSize());
        fileInfo.setType(type);
        fileInfo.setContentType(file.getContentType());
        LoginUser loginUser = SecurityUtils.getLoginUser();
        fileInfo.setCreateUser(loginUser.getUsername());
        fileInfo.setUrl("/file/" + type + "/" + originalFilename);

        // save to db
        fileManageMapper.insert(fileInfo);

        return fileInfo;
    }

    @Override
    public String upload(byte[] fileByte, String fileName, Integer type) {

        File dateDir = new File(localPath + type);
        if (!dateDir.exists()) {
            dateDir.mkdirs();
        }

        try {
            Files.write(Paths.get(dateDir.getPath() + "/" + fileName), fileByte);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        FileInfo fileInfo = new FileInfo();
        fileInfo.setFileName(fileName);
        fileInfo.setType(type);
        LoginUser loginUser = SecurityUtils.getLoginUser();
        fileInfo.setCreateUser(loginUser.getUsername());
        fileInfo.setUrl("/file/" + type + "/" + fileName);

        // save to db
        fileManageMapper.insert(fileInfo);

        return fileInfo.getUrl();
    }

    @Override
    public Page<FileInfo> search(PageParam<FileInfo> pageParam) {
        FileInfo fileParam = pageParam.getParams();
        Page<FileInfo> rowPage = new Page(pageParam.getPage(), pageParam.getSize());

        LambdaQueryWrapper<FileInfo> queryWrapper = new LambdaQueryWrapper<>();

        // 文件分类category(办公常用、系统工具)
        queryWrapper.eq(fileParam.getType() != null, FileInfo::getType, fileParam.getType());
        queryWrapper.eq(FileInfo::getDelFlag, 0);

        // 文件名称
        queryWrapper.like(StringUtils.isNotEmpty(fileParam.getFileName()), FileInfo::getFileName, fileParam.getFileName());
        // 文件拓展名
        queryWrapper.like(StringUtils.isNotEmpty(fileParam.getExtension()), FileInfo::getExtension, fileParam.getExtension());
        // ContentType
        queryWrapper.like(StringUtils.isNotEmpty(fileParam.getContentType()), FileInfo::getContentType, fileParam.getContentType());

        // 下载量排序
        queryWrapper.orderByDesc(FileInfo::getDownloads);

        return fileManageMapper.selectPage(rowPage, queryWrapper);
    }

    @Override
    public List<FileInfo> top(String type, Integer num) {
        List<String> list = Arrays.asList(type.split(","));
        return fileManageMapper.getTop(list, num);
    }

    @Override
    public void fileLog(String uri, String ip) {
        QueryWrapper<FileInfo> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotEmpty(uri), "url", uri);
        FileInfo fileInfo = fileManageMapper.selectOne(wrapper);
        fileInfo.setDownloads(fileInfo.getDownloads() + 1);

        fileManageMapper.updateById(fileInfo);

        // 添加日志
        SysLog sysLog = new SysLog();
        sysLog.setRemoteAddr(ip);
        sysLog.setRequestUri(uri);
//        LoginUser loginUser = SecurityUtils.getLoginUser();
//        sysLog.setCreateBy(loginUser);

        sysLog.setTitle("file_download");
        sysLog.setParams(JSON.toJSONString(fileInfo));

        sysLogServices.save(sysLog);

    }

    @Transactional
    @Override
    public boolean delete(Integer id) {
        // 删除磁盘文件
        FileInfo fileInfo = fileManageMapper.selectById(id);
        // DB删除
        try {
            int i = fileManageMapper.deleteById(fileInfo);
            boolean b = false;
            if (i == 1) {
                String path = localPath + fileInfo.getUrl().replaceFirst("/file/", "");
                b = Files.deleteIfExists(Paths.get(path));
                if (!b) {
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                }
            }

            return b;
        } catch (IOException e) {
            e.printStackTrace();
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }

    }
}
